/*
********************************************************************************
                                                                                
Software License Agreement                                                      
                                                                                
Copyright  2007-2008 Microchip Technology Inc.  All rights reserved.           
                                                                                
Microchip licenses to you the right to use, modify, copy and distribute Software
only when embedded on a Microchip microcontroller or digital signal controller  
that is integrated into your product or third party product (pursuant to the    
sublicense terms in the accompanying license agreement).                        
                                                                                
You should refer to the license agreement accompanying this Software for        
additional information regarding your rights and obligations.                   
                                                                                
SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,   
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF        
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.  
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER       
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR    
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES         
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR     
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF        
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES          
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.     
                                                                                
********************************************************************************
*/

#if defined(__PIC24F__)
    #include <p24fxxxx.h>
#elif defined(__18CXX)
    #include <p18cxxx.h>
#elif defined(__PIC32MX__)
    #include <p32xxxx.h>
    #include "plib.h"
#else
    #error No processor header file.
#endif

/**********************************************************************
 * LAB2 Step 1d - Define USB Support Mode (et. al.)                   *
 **********************************************************************
 **************************  B E G I N N E R  *************************
 **********************************************************************
 *  1.  "Un-commment" the "#define" statements below to select the USB support
 *      mode (and other configuration options) for the USB Firmware 
 *      Stack, select the USB interface's ping-pong buffering mode, and
 *      the number of entries in the TPL table. For a complete listing of
 *		available options, refer to AN1141 + the USB_HOST.h stack file
 */ 

// Define support mode

#define USB_SUPPORT_HOST

//Identify the ping pong mode

#define USB_PING_PONG_MODE  USB_PING_PONG__FULL_PING_PONG

// Identify the number of TPL entries

#define NUM_TPL_ENTRIES 1

// Define the supported transfer type(s)

#define USB_SUPPORT_INTERRUPT_TRANSFERS
#define USB_NUM_INTERRUPT_NAKS 3

#define USB_SUPPORT_BULK_TRANSFERS
#define USB_NUM_BULK_NAKS 20000

// Define the initial VBUS current (in units of 2mA)

#define USB_INITIAL_VBUS_CURRENT (100/2)

// Define the insert delay time for Vbus settling.

#define USB_INSERT_TIME             (250+1) // 250 ms

/*
 **********************************************************************
 * END LAB2 Step 1d                                                   *
 **********************************************************************/
 

/**********************************************************************
 * LAB2 Step 4h - Define the USBTasks() Macro                         *
 **********************************************************************
 **************************  B E G I N N E R  *************************
 **********************************************************************
 *  1.  "Un-comment" the lines below that call the "USBHostTasks()" and
 *      "USBHostGenericTasks()" routines from within the "USBTasks()" 
 *      macro.
 */
 
// Define the main USB state machine macro

#define USBTasks()                  \
    {                               \
        USBHostTasks();             \
        USBHostGenericTasks();      \
    }

// Define the macro to call the USB Host initializtion routine

#define USBInitialize(x)            \
    (                               \
        USBHostInit(x)              \
    )

/*
 **********************************************************************
 * END LAB2 Step 4h                                                   *
 **********************************************************************/


