/******************************************************************************

    USB Configuration Header

This file contains definitions used to configure the USB FW Stack.

 * Filename:        usb_config.c
 * Dependancies:    TBD
 * Processor:       TBD
 * Hardware:        TBD
 * Assembler:       TBD
 * Linker:          TBD
 * Company:         Microchip Technology, Inc.

Software License Agreement

The software supplied herewith by Microchip Technology Incorporated
(the Company) for its PICmicro Microcontroller is intended and
supplied to you, the Companys customer, for use solely and
exclusively on Microchip PICmicro Microcontroller products. The
software is owned by the Company and/or its supplier, and is
protected under applicable copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to
civil liability for the breach of the terms and conditions of this
license.

THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#if defined(__PIC24F__)
    #include <p24fxxxx.h>
#elif defined __PIC32MX__
    #include <p32xxxx.h>
    #include "plib.h"
#else
    #error No processor header file.
#endif

// Define support mode

#define USB_SUPPORT_HOST

//Identify the ping pong mode

#define USB_PING_PONG_MODE  USB_PING_PONG__FULL_PING_PONG

// Identify the number of TPL entries

#define NUM_TPL_ENTRIES     1


// Define the supported transfer type(s)

#define USB_SUPPORT_INTERRUPT_TRANSFERS
#define USB_NUM_INTERRUPT_NAKS              3

#define USB_SUPPORT_BULK_TRANSFERS
#define USB_NUM_BULK_NAKS                   20000

// Define the maximum current the device is allowed to draw from VBus

#define USB_INITIAL_VBUS_CURRENT    (100/2) // 100 mA

// Define the insert delay time for Vbus settling.

#define USB_INSERT_TIME             (250+1) // 250 ms

/**********************************************************************
 * LAB3 Step 1d - Enable Transfer Events                              *
 **********************************************************************
 **************************  B E G I N N E R  *************************
 **********************************************************************
 *  1.  "Un-comment" the line below to enable transfer events.
 */

// Enable transfer events
#define USB_ENABLE_TRANSFER_EVENT

/*
 **********************************************************************
 * END LAB3 Step 1d                                                   *
 **********************************************************************/


/**********************************************************************
 * LAB3 Step 2c - Identify the Application's Event Handler            *
 **********************************************************************
 **************************  B E G I N N E R  *************************
 **********************************************************************
 *  1.  "Un-comment" the line below to identify the application's
 *      event-handling routine.
 *  2.  Observe that there is no definition of "USBTasks()".
 */

// Identify the application's event handler
#define USB_HOST_APP_EVENT_HANDLER usb_custom_host_demo_EventHandler


/*
 **********************************************************************
 * END LAB3 Step 2c                                                   *
 **********************************************************************/




